/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.2.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for Dialog Semiconductor SC14CVMDECT-SF2 eval board
*/

#include "iosc14441.h"
#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define LED1_ON     (P2_SET_DATA_REG = (1u << 1))    // P2.1 D1
#define LED2_ON     (P2_SET_DATA_REG = (1u << 6))    // P2.6 D2
#define LED3_ON     (P3_SET_DATA_REG = (1u << 3))    // P3.3 D3
#define LED4_ON     (P3_SET_DATA_REG = (1u << 4))    // P3.4 D4
#define LED5_ON     (P3_SET_DATA_REG = (1u << 5))    // P3.5 D5
#define LED6_ON     (P3_SET_DATA_REG = (1u << 6))    // P3.6 D6
#define LED7_ON     (P3_SET_DATA_REG = (1u << 7))    // P3.7 D7
#define LED8_ON     (P1_SET_DATA_REG = (1u << 5))    // P1.5 D8

#define LED1_OFF    (P2_RESET_DATA_REG = (1u << 1))  // P2.1
#define LED2_OFF    (P2_RESET_DATA_REG = (1u << 6))  // P2.6
#define LED3_OFF    (P3_RESET_DATA_REG = (1u << 3))  // P3.3
#define LED4_OFF    (P3_RESET_DATA_REG = (1u << 4))  // P3.4
#define LED5_OFF    (P3_RESET_DATA_REG = (1u << 5))  // P3.5
#define LED6_OFF    (P3_RESET_DATA_REG = (1u << 6))  // P3.6
#define LED7_OFF    (P3_RESET_DATA_REG = (1u << 7))  // P3.7
#define LED8_OFF    (P1_RESET_DATA_REG = (1u << 5))  // P1.5

#define LED1_STATE  (P2_DATA_REG & (1u << 1))        // P2.1
#define LED2_STATE  (P2_DATA_REG & (1u << 6))        // P2.6
#define LED3_STATE  (P3_DATA_REG & (1u << 3))        // P3.3
#define LED4_STATE  (P3_DATA_REG & (1u << 4))        // P3.4
#define LED5_STATE  (P3_DATA_REG & (1u << 5))        // P3.5
#define LED6_STATE  (P3_DATA_REG & (1u << 6))        // P3.6
#define LED7_STATE  (P3_DATA_REG & (1u << 7))        // P3.7
#define LED8_STATE  (P1_DATA_REG & (1u << 5))        // P1.5

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // GPIO, Out P1.5
  //
  P1_MODE_REG       &= ~(unsigned short)0x00E0u;
  P1_DIR_REG        |= (unsigned short)(3u << 10);
  P1_RESET_DATA_REG  = (1u << 5);
  //
  // GPIO, Out P2.1, P2.6
  //
  P2_MODE_REG       &= ~(unsigned short)0x200Cu;
  P2_DIR_REG        |= (unsigned short)(3u << 2) | (3u << 12);
  P2_RESET_DATA_REG  = (1u << 1) | (1u << 6);
  //
  // GPIO, Out P3.3, P3.4, P3.5, P3.6, P3.7
  //
  P3_MODE_REG       &= ~(unsigned short)0xFFC0u;
  P3_DIR_REG        |= (unsigned short)((3u << 6) | (3u << 8) | (3u << 10) | (3u << 12) | (3u << 14));
  P3_RESET_DATA_REG  = (1u << 3) | (1u << 4) | (1u << 5) | (1u << 6) | (1u << 7);
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  switch(Index) {
    case 0:
      LED1_ON;
      break;
    case 1:
      LED2_ON;
      break;
    case 2:
      LED3_ON;
      break;
    case 3:
      LED4_ON;
      break;
    case 4:
      LED5_ON;
      break;
    case 5:
      LED6_ON;
      break;
    case 6:
      LED7_ON;
      break;
    case 7:
      LED8_ON;
      break;
    default:
      return;
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  switch(Index) {
    case 0:
      LED1_OFF;
      break;
    case 1:
      LED2_OFF;
      break;
    case 2:
      LED3_OFF;
      break;
    case 3:
      LED4_OFF;
      break;
    case 4:
      LED5_OFF;
      break;
    case 5:
      LED6_OFF;
      break;
    case 6:
      LED7_OFF;
      break;
    case 7:
      LED8_OFF;
      break;
    default:
      return;
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  unsigned short state;
  switch (Index) {
    case 0:
      state = LED1_STATE;
      break;
    case 1:
      state = LED2_STATE;
      break;
    case 2:
      state = LED3_STATE;
      break;
    case 3:
      state = LED4_STATE;
      break;
    case 4:
      state = LED5_STATE;
      break;
    case 5:
      state = LED6_STATE;
      break;
    case 6:
      state = LED7_STATE;
      break;
    case 7:
      state = LED8_STATE;
      break;
    default:
      return;
  }
  if(state != 0)
    BSP_ClrLED(Index);
  else
    BSP_SetLED(Index);
}

/*************************** End of file ****************************/
