/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.2.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "iosc14441.h"
#include "intrinsics.h"

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings (Timer 1 Block)
*/
#define OS_TIMER_FREQ  (1152000u)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

#define TIM1_INT_PEND      (1u << 8)
#define TIM1_INT_PRIORITY  1u

/*********************************************************************
*
*       Interrupt vector functions
*
**********************************************************************
*/
#pragma vector=0x00
static __interrupt void _ISR_00(void) {
    asm("excp bpt");
}
#pragma vector=NMI_INT
static __interrupt void _ISR_01(void) {
    asm("excp bpt");
}
#pragma vector=0x02
static __interrupt void _ISR_02(void) {
    asm("excp bpt");
}
#pragma vector=0x03
static __interrupt void _ISR_03(void) {
    asm("excp bpt");
}
#pragma vector=0x04
static __interrupt void _ISR_04(void) {
    asm("excp bpt");
}
#pragma vector=SVC_TRAP
static __interrupt void _ISR_05(void) {
    asm("excp bpt");
}
#pragma vector=DVZ_TRAP
static __interrupt void _ISR_06(void) {
    asm("excp bpt");
}
#pragma vector=FLG_TRAP
static __interrupt void _ISR_07(void) {
    asm("excp bpt");
}
#pragma vector=BPT_TRAP
static __interrupt void _ISR_08(void) {
    asm("excp bpt");
}
#pragma vector=TRC_TRAP
static __interrupt void _ISR_09(void) {
    asm("excp bpt");
}
#pragma vector=UND_TRAP
static __interrupt void _ISR_0A(void) {
    asm("excp bpt");
}
#pragma vector=0x0B
static __interrupt void _ISR_0B(void) {
    asm("excp bpt");
}
#pragma vector=IAD_TRAP
static __interrupt void _ISR_0C(void) {
    asm("excp bpt");
}
#pragma vector=0x0D
static __interrupt void _ISR_0D(void) {
    asm("excp bpt");
}
#pragma vector=DBG_TRAP
static __interrupt void _ISR_0E(void) {
    asm("excp bpt");
}
#pragma vector=ISE_INT
static __interrupt void _ISR_0F(void) {
    asm("excp bpt");
}
#pragma vector=ACCESS12_INT
static __interrupt void _ISR_10(void) {
    asm("excp bpt");
}
#pragma vector=KEYB_INT
static __interrupt void _ISR_11(void) {
    asm("excp bpt");
}
#pragma vector=UART2_RI_TI_INT
static __interrupt void _ISR_12(void) {
    asm("excp bpt");
}
#pragma vector=CT_CLASSD_INT
static __interrupt void _ISR_13(void) {
    asm("excp bpt");
}
#pragma vector=UART_RI_INT
static __interrupt void _ISR_14(void) {
    asm("excp bpt");
}
#pragma vector=UART_TI_INT
static __interrupt void _ISR_15(void) {
    asm("excp bpt");
}
#pragma vector=SPI_INT
static __interrupt void _ISR_16(void) {
    asm("excp bpt");
}
//#pragma vector=TIM0_INT
//static __interrupt void _ISR_17(void) {
//    asm("excp bpt");
//}
//#pragma vector=TIM1_INT
//static __interrupt void _ISR_18(void) {
//    asm("excp bpt");
//}
#pragma vector=CLK100_INT
static __interrupt void _ISR_19(void) {
    asm("excp bpt");
}
#pragma vector=DIP_INT
static __interrupt void _ISR_1A(void) {
    asm("excp bpt");
}
#pragma vector=AD_INT
static __interrupt void _ISR_1B(void) {
    asm("excp bpt");
}
#pragma vector=SPI2_INT
static __interrupt void _ISR_1C(void) {
    asm("excp bpt");
}
#pragma vector=DSP_INT
static __interrupt void _ISR_1D(void) {
    asm("excp bpt");
}
#pragma vector=ULP_INT
static __interrupt void _ISR_1E(void) {
    asm("excp bpt");
}
#pragma vector=0x1F
static __interrupt void _ISR_1F(void) {
    asm("excp bpt");
}

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return TIMER1_RELOAD_N_REG;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (RESET_INT_PENDING_REG & TIM1_INT_PEND);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#pragma vector=TIM1_INT
static __interrupt void SysTick_Handler(void) {
  RESET_INT_PENDING_REG = TIM1_INT_PEND;  // Reset Timer 1 interrupt flag
  WATCHDOG_REG = 0xFF;                    // WATCHDOG retrigger to be placed somewhere
  OS_INT_EnterNestable();
  OS_INT_EnterIntStack();
  OS_TICK_Handle();
  OS_INT_LeaveIntStack();
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_DOWNCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  volatile unsigned short i;
  OS_INT_IncDI();
  //
  // CLK_AMBA_SRAM1_EN | CLK_AMBA_PCLK_DIV_1 | CLK_AMBA_HCLK_DIV_1;
  //
  CLK_AMBA_REG                    = ((1u << 5) | (1u << 3) | (1u << 0));
  BAT_CTRL2_REG_bit.NTC_DISABLE   = 1u;
  CLK_XTAL_CTRL_REG_bit.XTAL_DIV  = 1u;   // 20.736 MHz crystal
  TEST_CTRL2_REG                 |= 0x40; // 20.736 MHz crystal
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Make sure that no one disturbs.
  //
  INT0_PRIORITY_REG     = 0u;
  INT1_PRIORITY_REG     = 0u;
  INT2_PRIORITY_REG     = 0u;
  INT3_PRIORITY_REG     = 0u;
  RESET_INT_PENDING_REG = 0xFFFFu;
  //
  // Disable/stop misc. units.
  //
  DIP_CTRL_REG_bit.URST   = 1u;
  DSP_CTRL_REG_bit.DSP_EN = 0u;
  //
  // Initialize Timer 1 to generate periodic interrupt = 1 msec
  //
  TIMER_CTRL_REG_bit.TIM1_CTRL        = 0u;
  TIMER1_RELOAD_M_REG                 = 1150u;
  TIMER1_RELOAD_N_REG                 = 0x0u;
  TIMER_CTRL_REG_bit.TIM1_MODE        = 0u;
  TIMER_CTRL_REG_bit.CLK_CTRL1        = 0u;
  TIMER_CTRL_REG_bit.CLK_DIV8         = 0u;
  TIMER_CTRL_REG_bit.TIM1_CTRL        = 1u;
  INT1_PRIORITY_REG_bit.TIM1_INT_PRIO = TIM1_INT_PRIORITY;
  RESET_FREEZE_REG_bit.FRZ_TIM1       = 1u;
  //
  // Enable Watchdog reset
  //
  TIMER_CTRL_REG_bit.WDOG_CTRL  = 1u;
  WATCHDOG_REG                  = 0xFFu;
  RESET_FREEZE_REG_bit.FRZ_WDOG = 1u;
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
    WATCHDOG_REG = 0xFF;                    // WATCHDOG retrigger to be placed somewhere
  }
}

/*********************************************************************
*
*       OS_BSP_GetCylces()
*
*  Function description
*    Returns the current system time in timer cycles as a 32-bit value.
*
*  Return value
*    The current system time in timer cycles as a 32-bit integer value.
*/
OS_U32 OS_BSP_GetCylces(void) {
  OS_U32 TickCnt;
  OS_U32 Cycles;

  OS_INT_IncDI();
  TickCnt= OS_TIME_GetTicks32();
  Cycles = 1150u - TIMER1_RELOAD_N_REG;
  //
  // Check if timer interrupt is pending
  //
  if ((RESET_INT_PENDING_REG & TIM1_INT_PEND) != 0u) {
    TickCnt++;
    Cycles = 1150u - TIMER1_RELOAD_N_REG;
  }
  OS_INT_DecRI();

  return (TickCnt * 1150u) + Cycles;
}

/*********************************************************************
*
*       OS_BSP_ConvertCycles2us()
*
*  Function description
*    Converts timer cycles into microseconds.
*
*  Parameters
*    Cycles: Timer cycles
*
*  Return value
*    The converted value in microseconds.
*/
OS_U32 OS_BSP_ConvertCycles2us(OS_U32 Cycles) {
  return ((Cycles * 1000u) / 1150u);
}

/*********************************************************************
*
*       OS_BSP_Convertus2Cycles()
*
*  Function description
*    Converts microseconds into timer cycles.
*
*  Parameters
*    us: Microseconds
*
*  Return value
*    The converted value in timer cycles
*/
OS_U32 OS_BSP_Convertus2Cycles(OS_U32 us) {
  return ((us * 1150u) / 1000u);
}

/*************************** End of file ****************************/
