/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.2.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for National Semiconductor CP3-DB-xx26-2 eval board
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define MCFG           (*(__data24 volatile unsigned int* )0xFFF910)
#define LED_LATCH      (*(__data24 volatile unsigned int* )0xFFFB78)
#define LED0_BIT       0
#define LED1_BIT       1

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  MCFG      |= 0x01u;  // Enable external bus
  LED_LATCH |= 0x00u;  // Disable all leds
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index < 2) {
    LED_LATCH |= (1uL << (LED0_BIT + Index));
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index < 2) {
    LED_LATCH &= ~(1uL << (LED0_BIT + Index));
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (LED_LATCH & (1 << (LED0_BIT + Index))) {
    BSP_ClrLED(Index);
  } else {
    BSP_SetLED(Index);
  }
}

/*************************** End of file ****************************/
