/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.10.2.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : RTOSInit_CP3UB26.c
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_PRESCALE (6u)
#define OS_TIMER_FREQ     (48000000u / OS_TIMER_PRESCALE)
#define OS_TICK_FREQ      (1000u)
#define OS_INT_FREQ       (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  #include "JLINKMEM.h"
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "BSP_UART.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (38400u)
#endif

#define IENAM0                 (*(__data24 volatile unsigned short* )0xFFFE0Eu)
#define TPRSC                  (*(__data24 volatile unsigned char*  )0xFFFF48u)
#define TCKC                   (*(__data24 volatile unsigned char*  )0xFFFF4Au)
#define TCNT1                  (*(__data24 volatile unsigned short* )0xFFFF40u)
#define TCRA                   (*(__data24 volatile unsigned short* )0xFFFF42u)
#define TCTRL                  (*(__data24 volatile unsigned char*  )0xFFFF4Cu)
#define TICTL                  (*(__data24 volatile unsigned char*  )0xFFFF4Eu)
#define TICLR                  (*(__data24 volatile unsigned char*  )0xFFFF50u)
#define SZCFG0                 (*(__data24 volatile unsigned short* )0xFFF904u)  // Bus and memory timing
#define WAIT_STATES_0          (1)
#define HOLD_STATES_0          (0)
#define HOLD_STATES_MASK       (0x3uL << 3)
#define WAIT_STATES_MASK       (0x7uL << 0)
#define SZCFG0_DEFAULT_VALUE   (0x069Fu)
#define SZCFG0_VALUE           ((SZCFG0_DEFAULT_VALUE & ~(HOLD_STATES_MASK | WAIT_STATES_MASK)) | (WAIT_STATES_0 << 0) | (HOLD_STATES_0 << 3) )
#define CRCTRL                 (*(__data24 volatile unsigned char*  )0xFFFC40u)  // PLL register
#define PRSFC                  (*(__data24 volatile unsigned char*  )0xFFFC42u)
#define PRSSC                  (*(__data24 volatile unsigned char*  )0xFFFC44u)
#define PRSAC                  (*(__data24 volatile unsigned char*  )0xFFFC46u)
#define PLL_MULTIPLIER_VALUE   (0x40u)
#define CRCTRL_FCLK            (0x02u)
#define CRCTRL_PLLWD           (0x04u)
#define TIMERA_IRQ             (23u)
#define UART0RX_IRQ            (33u)
#define UART0TX_IRQ            (23u)
#ifdef CSPY_SIM
  #define MASKINTS_START       (0u)
#else
  #define MASKINTS_START       (16u)
#endif

#define TIMER_CLOCK_EN_BIT     (0)                                               // Configurtion for timer, default is timer A
#define TIMER_INT_EN_BIT       (4)
#define TIMER_INT_PENDING_BIT  (0)
#define OS_TIM_IENAM           (IENAM0)
#define OS_TIM_IENAM_BIT       (14)

#define USART0_BASE_ADDR       (0xFFF200u)
#define UxRBUF                 (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x02u))
#define UxTBUF                 (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x00u))
#define UxPSR                  (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x0Eu))
#define UxBAUD                 (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x0Cu))
#define UxFRS                  (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x08u))
#define UxMDSL1                (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x0Au))
#define UxSTAT                 (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x06u))
#define UxICTRL                (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x04u))
#define UxOVR                  (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x10u))
#define UxMDSL2                (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x12u))
#define UxSPOS                 (*(__data24 volatile unsigned char* ) (USART0_BASE_ADDR + 0x14u))
#define PEALT                  (*(__data24 volatile unsigned int*  ) 0xFFFCC0u)

//
// Calculate values for Baudrate generator.
// Using other clock frequency than 48 MHz may require different default values
//
#define OVR_VALUE              (10u)         // Oversampling = 10
#define PSR_VALUE              (0x18u)       // Prescaler = 12.5
#define UART_PRESCALE          (125u / 10u)
#define BAUD_VALUE             ((OS_PCLK_UART / _UART_PRESCALE / _OVR_VALUE / OS_BAUDRATE) - 1)

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return TCNT1;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (TICTL & (1uL << TIMER_INT_PENDING_BIT));
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       SysTick_Handler()
*
*  Function description
*    This is the hardware timer exception handler.
*/
#pragma vector=23
static __interrupt void SysTick_Handler(void) {
  TICLR = (1 << TIMER_INT_PENDING_BIT);  // Reset Timer interrupt flag
  OS_INT_EnterNestable();
  OS_INT_EnterIntStack();
  OS_TICK_Handle();
  OS_INT_LeaveIntStack();
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_DOWNCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  //
  // Inform embOS about the timer settings
  //
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
  //
  // Enable PLL
  //
  PRSFC   = PLL_MULTIPLIER_VALUE;     // PLL multilplier set to 4, divisor set to 1
  CRCTRL &= ~CRCTRL_PLLWD;            // disable power down bit
  do {                                // wait until PLL ready
    CRCTRL &= ~CRCTRL_FCLK;
  } while ((CRCTRL & CRCTRL_FCLK) == CRCTRL_FCLK);
  //
  // Speedup memory interface, setup hold and wait states
  //
  SZCFG0  = SZCFG0_VALUE;
  //
  // Initialize Timer A to generate periodic interrupt = 1ms
  //
  TCTRL   = 0x82;                             // Enable timer unit and set mode=3
  TPRSC   = (OS_TIMER_PRESCALE - 1);          // Initialize clock divider
  TCKC   |= (1 << TIMER_CLOCK_EN_BIT);        // Enable clock
  TCRA    = (OS_TIMER_FREQ/OS_TICK_FREQ - 1); // Set reload value
  TICTL  |= (1 << TIMER_INT_EN_BIT);          // Enable timer interrupt
  OS_TIM_IENAM |= (1 << OS_TIM_IENAM_BIT);    // Enable interrupt controller for timer
  //
  // Initialize communication for embOSView
  //
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  UxFRS   = 0x00;
  UxMDSL1 = 0x00;
  UxICTRL = 0x40;                   // Enable Rx interrupt initially
  UxOVR   = _OVR_VALUE;
  UxMDSL2 = 0x00;

  UxPSR   = (_PSR_VALUE << 3) | ((_BAUD_VALUE >> 8) & 0x7);
  UxBAUD  = _BAUD_VALUE;
  PEALT   = 0xFF;
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
  }
}

/*********************************************************************
*
*       OS_BSP_GetCylces()
*
*  Function description
*    Returns the current system time in timer cycles as a 32-bit value.
*
*  Return value
*    The current system time in timer cycles as a 32-bit integer value.
*/
OS_U32 OS_BSP_GetCylces(void) {
  OS_U32 TickCnt;
  OS_U32 Cycles;

  OS_INT_IncDI();
  TickCnt= OS_TIME_GetTicks32();
  Cycles = TCNT1;
  //
  // Check if timer interrupt is pending
  //
  if ((TICTL & (1uL << TIMER_INT_PENDING_BIT)) != 0u) {
    TickCnt++;
    Cycles = TCNT1;
  }
  OS_INT_DecRI();

  return (TickCnt * (OS_TIMER_FREQ/OS_TICK_FREQ - 1)) + Cycles;
}

/*********************************************************************
*
*       OS_BSP_ConvertCycles2us()
*
*  Function description
*    Converts timer cycles into microseconds.
*
*  Parameters
*    Cycles: Timer cycles
*
*  Return value
*    The converted value in microseconds.
*/
OS_U32 OS_BSP_ConvertCycles2us(OS_U32 Cycles) {
  return ((Cycles * 1000u) / (OS_TIMER_FREQ/OS_TICK_FREQ - 1));
}

/*********************************************************************
*
*       OS_BSP_Convertus2Cycles()
*
*  Function description
*    Converts microseconds into timer cycles.
*
*  Parameters
*    us: Microseconds
*
*  Return value
*    The converted value in timer cycles
*/
OS_U32 OS_BSP_Convertus2Cycles(OS_U32 us) {
  return ((us * (OS_TIMER_FREQ/OS_TICK_FREQ - 1)) / 1000u);
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
/*********************************************************************
*
*       OS_Uart_RxHandler()
*       Uart0 Receive Interupt
*/
#pragma vector=(UART0RX_IRQ+MASKINTS_START)
static __interrupt void _OS_Uart_RxHandler(void) {
  unsigned int c;

  OS_INT_Enter();
  OS_INT_EnterIntStack();
  c = UxRBUF;
  //
  // Check if errors occured
  //
  if ( (UxSTAT & 0x08u) == 0x00u ) {
    OS_COM_OnRx(c);
  }
  OS_INT_LeaveIntStack();
  OS_INT_Leave();
}

/*********************************************************************
*
*       OS_Uart_TxHandler()
*       Uart0 Transmit Interupt
*/
#pragma vector=(UART0TX_IRQ+MASKINTS_START)
static __interrupt void _OS_Uart_TxHandler(void) {
  OS_INT_Enter();
  OS_INT_EnterIntStack();
  if (OS_COM_OnTx() != 0u) {  //  No more characters to send?
     UxICTRL &= ~0x20u;       //  Disable Tx Interrupt
  }
  OS_INT_LeaveIntStack();
  OS_INT_Leave();
}
#endif

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_JLINK)
  JLINKMEM_SendChar(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  UxTBUF   = c;            // Send character
  UxICTRL |= 0x20u;        // Enable Tx interrupt
#elif (OS_VIEW_IFSELECT == OS_VIEW_IF_ETHERNET)
  UDP_Process_Send1(c);
#elif (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)
  OS_USEPARA(c);           // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
#endif
}

/*************************** End of file ****************************/
